@@ -1,5 +1,7 @@
 from typing import Any, ClassVar, Generic, List, Optional, TypeVar, Union
 
+from typing_extensions import Self
+
 from pydantic import BaseModel, ConfigDict, Field, create_model, field_validator, model_validator, validator
 from pydantic.dataclasses import dataclass
 
@@ -291,11 +293,15 @@ def foo() -> None:
     class MyModel(BaseModel):
         number: int
         custom_validator = get_my_custom_validator('number')  # type: ignore[pydantic-field]
-# MYPY: error: Unused "type: ignore" comment
+# MYPY: error: Unused "type: ignore" comment  [unused-ignore]
 
         @model_validator(mode='before')
         @classmethod
-        def validate_values(cls, values: Any) -> Any:
+        def validate_before(cls, values: Any) -> Any:
             return values
 
+        @model_validator(mode='after')
+        def validate_after(self) -> Self:
+            return self
+
     MyModel(number=2)
