@@ -12,7 +12,7 @@ from typing import Any, Callable, ClassVar, Dict, FrozenSet, Generic, List, Opti
 
 import pytest
 from dirty_equals import HasRepr
-from pydantic_core import ArgsKwargs, SchemaValidator
+from pydantic_core import ArgsKwargs, CoreSchema, SchemaValidator, core_schema
 from typing_extensions import Annotated, Literal
 
 import pydantic
@@ -20,6 +20,7 @@ from pydantic import (
     BaseModel,
     ConfigDict,
     FieldValidationInfo,
+    GenerateSchema,
     PydanticDeprecatedSince20,
     PydanticUndefinedAnnotation,
     PydanticUserError,
@@ -810,7 +811,6 @@ def test_override_builtin_dataclass_2():
     assert f.seen_count == 7
 
 
-@pytest.mark.xfail(reason='Meta() is not being revalidated')
 def test_override_builtin_dataclass_nested():
     @dataclasses.dataclass
     class Meta:
@@ -2439,3 +2439,30 @@ def test_signature():
     assert str(inspect.signature(Model)) == (
         "(x: int, y: str = 'y', z: float = 1.0, a: float = <factory>, b: float = 1.0, c: float = <factory>) -> None"
     )
+
+
+def test_dataclasses_with_slots_and_default():
+    @pydantic.dataclasses.dataclass(slots=True)
+    class A:
+        a: int = 0
+
+    assert A().a == 0
+
+    @pydantic.dataclasses.dataclass(slots=True)
+    class B:
+        b: int = Field(1)
+
+    assert B().b == 1
+
+
+def test_schema_generator() -> None:
+    class LaxStrGenerator(GenerateSchema):
+        def str_schema(self) -> CoreSchema:
+            return core_schema.no_info_plain_validator_function(str)
+
+    @pydantic.dataclasses.dataclass
+    class Model:
+        x: str
+        __pydantic_config__ = ConfigDict(schema_generator=LaxStrGenerator)
+
+    assert Model(x=1).x == '1'
