@@ -24,7 +24,7 @@ from typing import (
 import pytest
 from dirty_equals import HasRepr, IsStr
 from pydantic_core import ErrorDetails, InitErrorDetails, PydanticSerializationError, core_schema
-from typing_extensions import Annotated, TypedDict, get_args
+from typing_extensions import Annotated, Literal, TypedDict, get_args
 
 from pydantic import (
     BaseModel,
@@ -446,7 +446,7 @@ def test_tuple_value_error():
         },
         {
             'type': 'decimal_parsing',
-            'loc': ('v', 2, 'function-after[to_decimal(), union[float,int,constrained-str]]'),
+            'loc': ('v', 2, 'function-after[to_decimal(), union[int,constrained-str,function-plain[str()]]]'),
             'msg': 'Input should be a valid decimal',
             'input': 'x',
         },
@@ -1237,9 +1237,7 @@ def test_multiple_errors():
             'type': 'is_instance_of',
             'loc': (
                 'a',
-                'function-after[check_digits_validator(), json-or-python[json=function-after[to_decimal(), '
-                'union[float,int,constrained-str]],python=lax-or-strict[lax=union[is-instance[Decimal],'
-                'function-after[to_decimal(), union[float,int,constrained-str]]],strict=is-instance[Decimal]]]]',
+                'function-after[check_digits_validator(), json-or-python[json=function-after[to_decimal(), union[int,constrained-str,function-plain[str()]]],python=lax-or-strict[lax=union[is-instance[Decimal],function-after[to_decimal(), union[int,constrained-str,function-plain[str()]]]],strict=is-instance[Decimal]]]]',
                 'is-instance[Decimal]',
             ),
             'msg': 'Input should be an instance of Decimal',
@@ -1250,10 +1248,8 @@ def test_multiple_errors():
             'type': 'decimal_parsing',
             'loc': (
                 'a',
-                'function-after[check_digits_validator(), json-or-python[json=function-after[to_decimal(), '
-                'union[float,int,constrained-str]],python=lax-or-strict[lax=union[is-instance[Decimal],'
-                'function-after[to_decimal(), union[float,int,constrained-str]]],strict=is-instance[Decimal]]]]',
-                'function-after[to_decimal(), union[float,int,constrained-str]]',
+                'function-after[check_digits_validator(), json-or-python[json=function-after[to_decimal(), union[int,constrained-str,function-plain[str()]]],python=lax-or-strict[lax=union[is-instance[Decimal],function-after[to_decimal(), union[int,constrained-str,function-plain[str()]]]],strict=is-instance[Decimal]]]]',
+                'function-after[to_decimal(), union[int,constrained-str,function-plain[str()]]]',
             ),
             'msg': 'Input should be a valid decimal',
             'input': 'foobar',
@@ -2563,3 +2559,24 @@ def test_multiple_enums():
         b: Optional[MyEnum]
 
     TypeAdapter(MyModel)
+
+
+@pytest.mark.parametrize(
+    ('literal_type', 'other_type', 'data', 'json_value', 'data_reversed', 'json_value_reversed'),
+    [
+        (Literal[False], str, False, 'false', False, 'false'),
+        (Literal[True], str, True, 'true', True, 'true'),
+        (Literal[False], str, 'abc', '"abc"', 'abc', '"abc"'),
+        (Literal[False], int, False, 'false', 0, '0'),
+        (Literal[True], int, True, 'true', 1, '1'),
+        (Literal[False], int, 42, '42', 42, '42'),
+    ],
+)
+def test_union_literal_with_other_type(literal_type, other_type, data, json_value, data_reversed, json_value_reversed):
+    class Model(BaseModel):
+        value: Union[literal_type, other_type]
+        value_types_reversed: Union[other_type, literal_type]
+
+    m = Model(value=data, value_types_reversed=data)
+    assert m.model_dump() == {'value': data, 'value_types_reversed': data_reversed}
+    assert m.model_dump_json() == f'{{"value":{json_value},"value_types_reversed":{json_value_reversed}}}'
